import 'dart:convert';

import 'package:fis_jsonrpc/rpc.dart';
import 'package:get/get.dart';
import 'package:platform_device_id/platform_device_id.dart';
import 'package:vitalapp/consts/devices.dart';
import 'package:vitalapp/global.dart';
import 'package:vitalapp/rpc.dart';
import 'package:vitalapp/store/store.dart';
import 'package:vnote_device_plugin/consts/types.dart';
import 'package:vitalapp/architecture/storage/text_storage.dart';
import 'package:vitalapp/managers/interfaces/models/device.dart';

import 'interfaces/device.dart';
import 'interfaces/system_setting.dart';

class DeviceManager extends IDeviceManager {
  static final _typeConvertMap = <String, String>{
    "Temp": DeviceTypes.TEMP,
    "Weight": DeviceTypes.WEIGHT,
    "SpO2": DeviceTypes.SPO2,
    "NIBP": DeviceTypes.NIBP,
    "GLU": DeviceTypes.SUGAR,
    "Urine": DeviceTypes.URINE,
    "ICReader": DeviceTypes.IC_READER,
    "ECG": DeviceTypes.HEART,
  };
  final _storage = TextStorage(fileName: "devices.cfg");
  final _deviceStorage = TextStorage(fileName: "devices.setting.cfg");
  List<DeviceModel>? _cacheList;

  /// TODO BAKA 获取设备权限缓存需要优化一下
  @override
  Future<List<String>> getAccessTypes() async {
    List<String> settingTypes = [];
    if (kIsOnline == true) {
      settingTypes =
          await Get.find<ISystemSettingManager>().getEnableDeviceKeysAsync();
      _deviceStorage.save(jsonEncode(settingTypes));
    } else {
      String? settingTypesString = await _deviceStorage.read();
      settingTypes =
          jsonDecode(settingTypesString ?? '').cast<String>().toList();
    }

    final types = settingTypes
        .map((e) => _typeConvertMap[e])
        .where((e) => e != null)
        .map((e) => e!)
        .toList();
    return types;
  }

  @override
  Future<void> addDevice(DeviceModel model) async {
    final source = await getDeviceList();
    final list = List<DeviceModel>.from(source);

    list.removeWhere((e) => e.type == model.type);
    list.add(model);

    saveDeviceList(list);
  }

  @override
  Future<void> removeDeviceByType(String type) async {
    final source = await getDeviceList();
    final list = List<DeviceModel>.from(source);

    list.removeWhere((e) => e.type == type);

    saveDeviceList(list);
  }

  @override
  Future<List<DeviceModel>> getDeviceList() async {
    if (_cacheList != null) return _cacheList!;

    final jsonText = await _storage.read();
    if (jsonText == null) {
      _cacheList = [];
    } else {
      final List<dynamic> mapList = jsonDecode(jsonText);
      _cacheList = mapList.map((e) => DeviceModel.fromJson(e)).toList();
    }
    return _cacheList!;
  }

  @override
  Future<void> saveDeviceList(List<DeviceModel> deviceList) async {
    _cacheList = deviceList;
    final jsonText = jsonEncode(deviceList);
    await _storage.save(jsonText);
  }

  Future<bool> reportAddDevice(DeviceModel model) async {
    try {
      if (kIsOnline) {
        final deviceId = await PlatformDeviceId.getDeviceId;
        final deviceKey4Server = DeviceConsts.transType2ServerKey(model.type);
        final request = CreatePeripheralDeviceRequest(
          token: Store.user.token,
          deviceCode: deviceId,
          enable: true,
          key: deviceKey4Server,
          name: model.productName,
        );
        final result = await rpc.device.createPeripheralDeviceAsync(request);
        return result;
      } else {
        //
        return true;
      }
    } catch (e) {
      return false;
    }
  }
}
