import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vnote_device_plugin/devices/base.dart';
import 'package:vitalapp/managers/interfaces/permission.dart';
import 'package:vitalapp/pages/medical/models/worker.dart';
import 'package:fis_common/logger/logger.dart';

abstract class BaseDeviceController<T extends DeviceWorkerBase>
    extends GetxController {
  BaseDeviceController(this.model, this.mac) {
    worker = createWorker();
  }
  late final T worker;
  late WorkerStatus connectStatus = WorkerStatus.connecting;
  final String model;
  final String mac;
  var permissionManager = Get.find<IPermissionManager>();

  @override
  void onInit() {
    getPermission();
    loadListeners();
    connect();
    super.onInit();
  }

  @protected
  T createWorker();

  @override
  void dispose() {
    disconnect();
    releaseListeners();
    super.dispose();
  }

  @protected
  @mustCallSuper
  void loadListeners() {
    worker.connectErrorEvent.addListener(_onConnectFail);
    worker.connectedEvent.addListener(_onConnectSuccess);
    worker.disconnectedEvent.addListener(_onDisconnected);
  }

  @protected
  void releaseListeners() {
    worker.connectErrorEvent.removeListener(_onConnectFail);
    worker.connectedEvent.removeListener(_onConnectSuccess);
    worker.disconnectedEvent.removeListener(_onDisconnected);
  }

  Future<void> connect() async {
    print("开始连接 mac: $mac, model: $model");
    logger.i("开始连接 mac: $mac, model: $model");
    await worker.connect();
  }

  Future<void> disconnect() async {
    print("断开连接 mac: $mac, model: $model");
    logger.i("断开连接 mac: $mac, model: $model");
    await worker.disconnect();
    // 处理设备关闭成功事件
  }

  Future<void> getPermission() async {
    await permissionManager.requestLocationPermission();
    await permissionManager.requestBluetoothConnectPermission();
    await permissionManager.requestBluetoothAdvertisePermission();
    await permissionManager.requestBluetoothScanPermission();
  }

  void _onConnectFail(_, dynamic error) {
    print("连接失败 mac: $mac, model: $model");
    logger.i("连接失败 mac: $mac, model: $model");
    connectStatus = WorkerStatus.connectionFailed;
    // 处理连接失败事件
  }

  void _onConnectSuccess(_, dynamic data) {
    logger.i("连接成功 mac: $mac, model: $model");
    print("连接成功 mac: $mac, model: $model");
    connectStatus = WorkerStatus.connected;
    // 处理连接成功事件
  }

  void _onDisconnected(_, dynamic error) {
    logger.i("连接中断 mac: $mac, model: $model");
    print("连接中断 mac: $mac, model: $model");

    connectStatus = WorkerStatus.disconnected;
    // 处理连接中断事件
  }

  /// 特殊处理检查值
  dynamic handleExamValueSpecially(num value, DynamicParameterDTO? param) {
    if (param == null) return value;

    final type = param.type;
    if (type == "even") {
      // 值保持偶数
      if (value % 2 == 0) {
        return value;
      }
      if (param.mode == "down") {
        return value -= 1; // 减1 凑偶数
      } else if (param.mode == "up") {
        return value += 1; // 加1 凑偶数
      }
    } else if (type == "odd") {
      // 值保持奇数
      if (value % 2 != 0) {
        return value;
      }
      if (param.mode == "down") {
        return value -= 1; // 减1 凑偶数
      } else if (param.mode == "up") {
        return value += 1; // 加1 凑偶数
      }
    }
    return value;
  }
}
